@@ -4,13 +4,15 @@
 #
 # Author: Paul McCarthy <pauldmccarthy@gmail.com>
 #
-"""This module provides the :class:`Action` and :class:`ToggleAction` classes.
-See the :mod:`.actions` package documentation for more details.
+"""This module provides the :class:`Action`, :class:`NeedOverlayAction`, and
+:class:`ToggleAction` classes.  See the :mod:`.actions` package documentation
+for more details.
 """
 
 
 import logging
 
+import fsl.data.image  as fslimage
 import fsleyes_props   as props
 import fsleyes_widgets as fwidgets
 
@@ -22,7 +24,6 @@ class ActionDisabledError(Exception):
     """Exception raised when an attempt is made to call a disabled
     :class:`Action`.
     """
-    pass
 
 
 class BoundWidget(object):
@@ -76,17 +77,68 @@ class Action(props.HasProperties):
     """
 
 
-    def __init__(self, func, instance=None):
+    @staticmethod
+    def title():
+        """May be overridden by sub-classes. Returns a title to be used
+        in menus.
+        """
+        return None
+
+
+    @staticmethod
+    def supportedViews():
+        """May be overridden to declare that this Action should be associated
+        with a specific :class:`.ViewPanel`. If overridden, must return a
+        list containing all of the supported ``ViewPanel`` types.
+        """
+        return None
+
+
+    @staticmethod
+    def ignoreTool():
+        """Used by the FSLeyes :mod:`.plugins` module for actions which are
+        loaded as plugins. Can be used to tell the ``plugins`` module that
+        a particular ``Action`` should not be added as an option to the
+        FSLeyes Tools menu.
+
+        Note that this method must be implemented on the class that is to
+        be ignored - inherited implementations from base classes are not
+        considered.
+        """
+        return False
+
+
+    def __init__(self,
+                 overlayList,
+                 displayCtx,
+                 func,
+                 name=None):
         """Create an ``Action``.
 
-        :arg func:     The action function.
+        :arg overlayList: The :class:`.OverlayList`.
 
-        :arg instance: Object associated with the function, if this ``Action``
-                       is encapsulating an instance method.
+        :arg displayCtx:  The :class:`.DisplayContext` associated with this
+                          ``Action``; note that this is not necessarily the
+                          master :class:`.DisplayContext`.
+
+        :arg func:        The action function.
+
+        :arg name:        Action name. Defaults to ``func.__name__``.
+
+        .. note:: If an ``Action`` encapsulates a method of an
+                  :class:`.ActionProvider` instance, it is assumed that the
+                  ``name`` is the name of the method on the instance.
         """
-        self.__instance     = instance
+
+        if name is None:
+            name = func.__name__
+
+        self.__overlayList  = overlayList
+        self.__displayCtx   = displayCtx
         self.__func         = func
-        self.__name         = func.__name__
+        self.__name         = '{}_{}'.format(type(self).__name__, id(self))
+        self.__actionName   = name
+        self.__destroyed    = False
         self.__boundWidgets = []
 
         self.addListener('enabled',
@@ -104,11 +156,36 @@ class Action(props.HasProperties):
         return self.__str__()
 
 
+    @property
+    def actionName(self):
+        """Returns the name of this ``Action``, often the method name of the
+        :class:`.ActionProvider` that implements the action. Not to be
+        confused with :meth:`name`.
+        """
+        return self.__actionName
+
+
+    @property
     def name(self):
-        """Returns the name of this ``Action``. """
+        """Not to be confused with :meth:`actionName`.
+        Returns a unique name for a specific ``Action`` instance, which
+        can be used (e.g.) for registering property listeners.
+        """
         return self.__name
 
 
+    @property
+    def overlayList(self):
+        """Return a reference to the :class:`.OverlayList`. """
+        return self.__overlayList
+
+
+    @property
+    def displayCtx(self):
+        """Return a reference to the :class:`.DisplayContext`. """
+        return self.__displayCtx
+
+
     def __call__(self, *args, **kwargs):
         """Calls this action. An :exc:`ActionDisabledError` will be raised
         if :attr:`enabled` is ``False``.
@@ -118,21 +195,26 @@ class Action(props.HasProperties):
             raise ActionDisabledError('Action {} is disabled'.format(
                 self.__name))
 
-        log.debug('Action {}.{} called'.format(
-            type(self.__instance).__name__,
-            self.__name))
-
-        if self.__instance is not None:
-            args = [self.__instance] + list(args)
+        log.debug('Action %s called', self.__name)
 
         return self.__func(*args, **kwargs)
 
 
+    @property
+    def destroyed(self):
+        """Returns ``True`` if :meth:`destroy` has been called, ``False``
+        otherwise.
+        """
+        return self.__destroyed
+
+
     def destroy(self):
         """Must be called when this ``Action`` is no longer needed. """
         self.unbindAllWidgets()
-        self.__func     = None
-        self.__instance = None
+        self.__destroyed   = True
+        self.__overlayList = None
+        self.__displayCtx  = None
+        self.__func        = None
 
 
     def bindToWidget(self, parent, evType, widget, wrapper=None):
@@ -231,12 +313,23 @@ class ToggleAction(Action):
 
 
     def __init__(self, *args, **kwargs):
-        """Create a ``ToggleAction``. All arguments are passed to
-        :meth:`Action.__init__`.
+        """Create a ``ToggleAction``.
+
+        :arg autoToggle: Must be specified as a keyword argument. If ``True``
+                         (the default), the state of ``toggled`` is inverted
+                         every time this action is called. Otherwise, the
+                         state of ``toggled``, and of all bound widgets/menu
+                         items, needs to be changed manually.
+
+        All other arguments are passed to :meth:`Action.__init__`.
         """
 
+        autoToggle = kwargs.pop('autoToggle', True)
+
         Action.__init__(self, *args, **kwargs)
 
+        self.__autoToggle = autoToggle
+
         self.addListener('toggled',
                          'ToggleAction_{}_internal'.format(id(self)),
                          self.__toggledChanged)
@@ -250,9 +343,18 @@ class ToggleAction(Action):
         # Copy the toggled value before running
         # the action, in case it gets inadvertently
         # changed
-        toggled      = self.toggled
-        result       = Action.__call__(self, *args, **kwargs)
-        self.toggled = not toggled
+        toggled = self.toggled
+        result  = Action.__call__(self, *args, **kwargs)
+
+        # Update self.toggled to align
+        # it with the widget state.
+        if self.__autoToggle:
+            self.toggled = not toggled
+
+        # Or update the widget state to
+        # align it with self.toggled
+        else:
+            self.__toggledChanged()
 
         return result
 
@@ -300,5 +402,61 @@ class ToggleAction(Action):
 
                 self.__setState(bw.widget)
 
-            except:
+            except Exception:
                 self.unbindWidget(bw.widget)
+
+
+class NeedOverlayAction(Action):
+    """The ``NeedOverlayAction`` is a convenience base class for actions
+    which can only be executed when an overlay of a specific type is selected.
+    It enables/disables itself based on the type of the currently selected
+    overlay.
+    """
+
+
+    def __init__(self,
+                 overlayList,
+                 displayCtx,
+                 func=None,
+                 overlayType=fslimage.Image):
+        """Create a ``NeedOverlayAction``.
+
+        :arg overlayList: The :class:`.OverlayList`.
+        :arg displayCtx:  The :class:`.DisplayContext`.
+        :arg func:        The action function
+        :arg overlayType: The required overlay type (defaults to :class:`.Image`)
+        """
+        Action.__init__(self, overlayList, displayCtx, func)
+
+        self.__overlayType = overlayType
+        self.__name        = 'NeedOverlayAction_{}_{}'.format(
+            type(self).__name__, id(self))
+
+        displayCtx .addListener('selectedOverlay',
+                                self.__name,
+                                self.__selectedOverlayChanged)
+        overlayList.addListener('overlays',
+                                self.__name,
+                                self.__selectedOverlayChanged)
+
+        self.__selectedOverlayChanged()
+
+
+    def destroy(self):
+        """Removes listeners from the :class:`.DisplayContext` and
+        :class:`.OverlayList`, and calls :meth:`.Action.destroy`.
+        """
+        self.displayCtx .removeListener('selectedOverlay', self.__name)
+        self.overlayList.removeListener('overlays',        self.__name)
+        Action.destroy(self)
+
+
+    def __selectedOverlayChanged(self, *a):
+        """Called when the selected overlay, or overlay list, changes.
+
+        Enables/disables this action depending on the nature of the selected
+        overlay.
+        """
+        ovl          = self.displayCtx.getSelectedOverlay()
+        ovlType      = self.__overlayType
+        self.enabled = (ovl is not None) and isinstance(ovl, ovlType)
