@@ -12,12 +12,15 @@ control* panel which allows the user to change overlay display settings.
 import logging
 import functools
 import collections
+import collections.abc as abc
+
+import wx
 
 import fsleyes_props                  as props
-
+import fsleyes.views.canvaspanel      as canvaspanel
+import fsleyes.controls.controlpanel  as ctrlpanel
 import fsleyes.strings                as strings
 import fsleyes.tooltips               as fsltooltips
-import fsleyes.panel                  as fslpanel
 
 from . import overlaydisplaywidgets   as odwidgets
 
@@ -25,8 +28,8 @@ from . import overlaydisplaywidgets   as odwidgets
 log = logging.getLogger(__name__)
 
 
-class OverlayDisplayPanel(fslpanel.FSLeyesSettingsPanel):
-    """The ``OverlayDisplayPanel`` is a :Class:`.FSLeyesPanel` which allows
+class OverlayDisplayPanel(ctrlpanel.SettingsPanel):
+    """The ``OverlayDisplayPanel`` is a :class:`.SettingsPanel` which allows
     the user to change the display settings of the currently selected
     overlay (which is defined by the :attr:`.DisplayContext.selectedOverlay`
     property). The display settings for an overlay are contained in the
@@ -55,23 +58,41 @@ class OverlayDisplayPanel(fslpanel.FSLeyesSettingsPanel):
     """
 
 
-    def __init__(self, parent, overlayList, displayCtx, frame):
+    @staticmethod
+    def supportedViews():
+        """Overrides :meth:`.ControlMixin.supportedViews`. The
+        ``OverlayDisplayPanel`` is only intended to be added to
+        :class:`.OrthoPanel`, :class:`.LightBoxPanel`, or
+        :class:`.Scene3DPanel` views.
+        """
+        return [canvaspanel.CanvasPanel]
+
+
+    @staticmethod
+    def defaultLayout():
+        """Returns a dictionary containing layout settings to be passed to
+        :class:`.ViewPanel.togglePanel`.
+        """
+        return {'location' : wx.LEFT}
+
+
+    def __init__(self, parent, overlayList, displayCtx, canvasPanel):
         """Create an ``OverlayDisplayPanel``.
 
         :arg parent:      The :mod:`wx` parent object.
         :arg overlayList: The :class:`.OverlayList` instance.
         :arg displayCtx:  The :class:`.DisplayContext` instance.
-        :arg frame:       The :class:`.FSLeyesFrame` instance.
+        :arg canvasPanel: The :class:`.CanvasPanel` instance.
         """
 
         from fsleyes.views.scene3dpanel import Scene3DPanel
 
-        fslpanel.FSLeyesSettingsPanel.__init__(self,
-                                               parent,
-                                               overlayList,
-                                               displayCtx,
-                                               frame,
-                                               kbFocus=True)
+        ctrlpanel.SettingsPanel.__init__(self,
+                                         parent,
+                                         overlayList,
+                                         displayCtx,
+                                         canvasPanel,
+                                         kbFocus=True)
 
         displayCtx .addListener('selectedOverlay',
                                  self.name,
@@ -81,7 +102,7 @@ class OverlayDisplayPanel(fslpanel.FSLeyesSettingsPanel):
                                  self.__selectedOverlayChanged)
 
         self.__threedee       = isinstance(parent, Scene3DPanel)
-        self.__viewPanel      = parent
+        self.__viewPanel      = canvasPanel
         self.__widgets        = None
         self.__currentOverlay = None
 
@@ -91,7 +112,7 @@ class OverlayDisplayPanel(fslpanel.FSLeyesSettingsPanel):
     def destroy(self):
         """Must be called when this ``OverlayDisplayPanel`` is no longer
         needed. Removes property listeners, and calls the
-        :meth:`.FSLeyesPanel.destroy` method.
+        :meth:`.SettingsPanel.destroy` method.
         """
 
         self.displayCtx .removeListener('selectedOverlay', self.name)
@@ -108,7 +129,7 @@ class OverlayDisplayPanel(fslpanel.FSLeyesSettingsPanel):
         self.__widgets        = None
         self.__currentOverlay = None
 
-        fslpanel.FSLeyesPanel.destroy(self)
+        ctrlpanel.SettingsPanel.destroy(self)
 
 
     def __selectedOverlayChanged(self, *a):
@@ -181,7 +202,7 @@ class OverlayDisplayPanel(fslpanel.FSLeyesSettingsPanel):
         allWidgets = self.__widgets.items()
         allWidgets = functools.reduce(lambda a, b: a + b, allWidgets)
 
-        fslpanel.FSLeyesSettingsPanel.setNavOrder(self, allWidgets)
+        ctrlpanel.SettingsPanel.setNavOrder(self, allWidgets)
 
 
     def __ovlTypeChanged(self, *a):
@@ -236,10 +257,13 @@ class OverlayDisplayPanel(fslpanel.FSLeyesSettingsPanel):
 
         if groupName == '3d':
             dispProps = odwidgets.get3DPropertyList(target)
-            dispSpecs = odwidgets.get3DWidgetSpecs( target)
+            dispSpecs = odwidgets.get3DWidgetSpecs( target, self.displayCtx)
         else:
-            dispProps = odwidgets.getPropertyList(target, self.__threedee)
-            dispSpecs = odwidgets.getWidgetSpecs( target, self.__threedee)
+            dispProps = odwidgets.getPropertyList(target,
+                                                  self.__threedee)
+            dispSpecs = odwidgets.getWidgetSpecs( target,
+                                                  self.displayCtx,
+                                                  self.__threedee)
 
         allLabels     = []
         allTooltips   = []
@@ -269,7 +293,7 @@ class OverlayDisplayPanel(fslpanel.FSLeyesSettingsPanel):
                     self.displayCtx,
                     self.__threedee)
 
-                if isinstance(container, collections.Sequence):
+                if isinstance(container, abc.Sequence):
                     specs    = container
                     keys     = [s.key for s in specs]
                     labels   = [strings.properties.get((target, k), None)
