@@ -13,14 +13,16 @@ data from :class:`.Nifti` overlays.
 import numpy     as np
 import OpenGL.GL as gl
 
-import fsl.utils.transform                 as transform
+import fsl.data.image                      as fslimage
+import fsl.transform.affine                as affine
 import fsl.utils.memoize                   as memoize
 
 import fsleyes.displaycontext.volume3dopts as volume3dopts
 import fsleyes.colourmaps                  as fslcmaps
 import fsleyes.gl.globject                 as globject
+import fsleyes.gl.textures                 as textures
+import fsleyes.gl.resources                as glresources
 import fsleyes.gl.routines                 as glroutines
-import fsleyes.gl.trimesh                  as trimesh
 
 
 class GLImageObject(globject.GLObject):
@@ -50,11 +52,13 @@ class GLImageObject(globject.GLObject):
     """
 
 
-    def __init__(self, overlay, displayCtx, canvas, threedee):
+    def __init__(self, overlay, overlayList, displayCtx, canvas, threedee):
         """Create a ``GLImageObject``.
 
         :arg image:       A :class:`.Nifti` object.
 
+        :arg overlayList: The :class`.OverlayList`.
+
         :arg displayCtx:  The :class:`.DisplayContext` object managing the
                           scene.
 
@@ -63,7 +67,8 @@ class GLImageObject(globject.GLObject):
         :arg threedee:    Set up for 2D or 3D rendering.
         """
 
-        globject.GLObject.__init__(self, overlay, displayCtx, canvas, threedee)
+        globject.GLObject.__init__(
+            self, overlay, overlayList, displayCtx, canvas, threedee)
 
         self.__name = 'GLImageObject_{}'.format(self.name)
 
@@ -125,6 +130,7 @@ class GLImageObject(globject.GLObject):
         globject.GLObject.destroy(self)
 
 
+    @property
     def destroyed(self):
         """Returns ``True`` if :meth:`destroy` has been called, ``False``
         otherwise.
@@ -255,7 +261,7 @@ class GLImageObject(globject.GLObject):
         if not hasattr(opts, 'interpolation') or opts.interpolation == 'none':
             voxCoords = opts.roundVoxels(voxCoords, daxes=[zax])
 
-        texCoords = transform.transform(voxCoords, v2tMat)
+        texCoords = affine.transform(voxCoords, v2tMat)
 
         return vertices, voxCoords, texCoords
 
@@ -287,7 +293,7 @@ class GLImageObject(globject.GLObject):
             d2vMat,
             bbox=bbox)
 
-        texCoords = transform.transform(voxCoords, v2tMat)
+        texCoords = affine.transform(voxCoords, v2tMat)
 
         return vertices, voxCoords, texCoords
 
@@ -334,10 +340,8 @@ class GLImageObject(globject.GLObject):
 
         if opts.transform == 'id':
             resolution = [1, 1, 1]
-        elif opts.transform in ('pixdim', 'pixdim-flip'):
-            resolution = image.pixdim[:3]
         else:
-            resolution = [min(image.pixdim[:3])] * 3
+            resolution = image.pixdim[:3]
 
         voxels = glroutines.pointGrid(
             image.shape,
@@ -350,7 +354,7 @@ class GLImageObject(globject.GLObject):
         voxels[:, zax] = zpos
 
         if space == 'voxel':
-            voxels = transform.transform(voxels, d2vMat)
+            voxels = affine.transform(voxels, d2vMat)
             voxels = opts.roundVoxels(voxels,
                                       daxes=[zax],
                                       roundOther=False)
@@ -381,7 +385,7 @@ class GLImageObject(globject.GLObject):
 
         if space == 'voxel':
             pass
-            # voxels = transform.transform(voxels, d2vMat)
+            # voxels = affine.transform(voxels, d2vMat)
             # voxels = opts.roundVoxels(voxels)
 
         return voxels
@@ -407,17 +411,25 @@ class GLImageObject(globject.GLObject):
         a 1D ``numpy`` array containing vertex indices.
 
         See the :meth:`get3DClipPlane` and :meth:`drawClipPlanes` methods.
+
+        .. note:: This method depends on the ``trimesh`` library - if it is
+                  not present, two empty arrays are returned.
         """
 
         origin, normal = self.get3DClipPlane(planeIdx)
         vertices       = self.generateVertices3D(bbox=bbox)[0]
         indices        = np.arange(vertices.shape[0]).reshape(-1, 3)
 
+        try:
+            import trimesh
+            import trimesh.intersections as tmint
+        except ImportError:
+            return np.zeros((0, 3)), np.zeros((0,))
+
         # Calculate the intersection of the clip
         # plane with the image bounding box
-        lines = trimesh.mesh_plane(
-            vertices,
-            indices,
+        lines = tmint.mesh_plane(
+            trimesh.Trimesh(vertices, indices),
             plane_normal=normal,
             plane_origin=origin)
 
@@ -458,7 +470,7 @@ class GLImageObject(globject.GLObject):
                 continue
 
             if xform is not None:
-                verts = transform.transform(verts, xform)
+                verts = affine.transform(verts, xform)
 
             verts = np.array(verts.ravel('C'), dtype=np.float32, copy=False)
 
@@ -502,3 +514,254 @@ class GLImageObject(globject.GLObject):
         """
         self.get3DClipPlane   .invalidate()
         self.clipPlaneVertices.invalidate()
+
+
+class AuxImageTextureManager:
+    """Utility class used by some :class:`GLImageObject` instances.
+
+    The ``AuxImageTextureManager`` is used to manage "auxillary"
+    :class:`.ImageTexture` instances which are used when rendering the
+    ``GLImageObject``. For example, :class:`.GLVolume` instances may need to
+    use an ``ImageTexture`` to store the data for the
+    :attr:`.VolumeOpts.clipImage` setting.
+    """
+
+
+    def __init__(self, globj, **auximages):
+        """Create an ``AuxImageTextureManager``.
+
+        Note that an initial value *must* be given for each auxillary texture
+        type.
+
+        :arg globj:     The :class:`GLImageObject` which requires the
+                        auxillary image textures.
+
+        :arg auximages: ``auxtype=initial_value`` for each auxillary image
+                        texture type. The initial value must be one of:
+
+                         - an :class:`.Image`
+                         - ``None``
+                         - A tuple containing an ``Image``, and a dict
+                           containing settings to initialise the
+                           ``ImageTexture`` (passed as ``kwargs`` to
+                           ``ImageTexture.__init__``).
+        """
+
+        self.__name        = '{}_{}'.format(type(self).__name__, id(self))
+        self.__globj       = globj
+        self.__image       = globj.image
+        self.__opts        = globj.opts
+        self.__displayCtx  = globj.displayCtx
+        self.__auxtypes    = tuple(auximages.keys())
+        self.__auxopts     = {t : None for t in self.__auxtypes}
+        self.__auximages   = {t : None for t in self.__auxtypes}
+        self.__auxtextures = {t : None for t in self.__auxtypes}
+
+        for which, image in auximages.items():
+            if isinstance(image, tuple):
+                image, settings = image
+            else:
+                settings = {}
+            self.registerAuxImage(which, image, **settings)
+
+
+    def destroy(self):
+        """Must be calld when this ``AuxImageTextureManager`` is no longer
+        needed. Clears references and destroys texture objects.
+        """
+        self.__globj      = None
+        self.__displayCtx = None
+        self.__opts       = None
+
+        for t in self.__auxtypes:
+            self.deregisterAuxImage(t, False)
+            self.__destroyAuxTexture(t)
+
+
+    @property
+    def name(self):
+        return self.__name
+
+
+    @property
+    def globj(self):
+        return self.__globj
+
+
+    @property
+    def image(self):
+        return self.__image
+
+
+    @property
+    def opts(self):
+        return self.__opts
+
+
+    @property
+    def displayCtx(self):
+        return self.__displayCtx
+
+
+    def texture(self, which):
+        return self.__auxtextures[which]
+
+
+    def textureXform(self, which):
+        """Generates and returns a transformation matrix which can be used to
+        transform texture coordinates from the main image to the specified
+        auxillary image.
+        """
+        opts     = self.opts
+        auximage = self.__auximages[which]
+        auxopts  = self.__auxopts[  which]
+
+        if auximage is None:
+            return np.eye(4)
+        else:
+            return affine.concat(
+                auxopts.getTransform('display', 'texture'),
+                opts   .getTransform('texture', 'display'))
+
+
+    def texturesReady(self):
+        """Returns ``True`` if all auxillary textures are in a usable
+        state, ``False`` otherwise.
+        """
+        for tex in self.__auxtextures.values():
+            if (tex is None) or (not tex.ready()):
+                return False
+        return True
+
+
+    def registerAuxImage(self, which, image, **kwargs):
+        """Register an auxillary image.
+
+        Creates an :class:`.ImageTexture` to store the image data.
+        Registers a listener with the :attr:`.NiftiOpts.volume` property of
+        the image, so the texture can be updated when the image volume
+        changes.
+
+        :arg which: Name of the auxillary image
+        :arg image: :class:`.Image` object
+
+        All other arguments are passed through to the :meth:`refreshAuxTexture`
+        method.
+        """
+
+        if self.__auximages[which] is not None:
+            self.deregisterAuxImage(which, False)
+
+        if image is None:
+            opts = None
+
+        else:
+            opts = self.displayCtx.getOpts(image)
+            def volumeChange(*a):
+                tex = self.texture(which)
+                tex.set(volume=opts.index()[3:])
+
+            opts.addListener('volume',
+                             '{}_{}'.format(self.name, which),
+                             volumeChange,
+                             weak=False)
+
+        self.__auximages[which] = image
+        self.__auxopts[  which] = opts
+        self.refreshAuxTexture(which, **kwargs)
+
+
+    def deregisterAuxImage(self, which, refreshTexture=True):
+        """De-register an auxillary image.  Deregisters the
+        :attr:`.NiftiOpts.volume` listener that was registered in
+        :meth:`registerAuxImage`, and destroys the associated
+        :class:`.ImageTexture`.
+
+        :arg which:          Name of the auxillary image
+
+        :arg refreshTexture: Defaults to ``True``. Call
+                             :meth:`refreshAuxTexture` to destroy the
+                             associated ``ImageTexture``.
+        """
+
+        image = self.__auximages[which]
+        opts  = self.__auxopts[  which]
+
+        if image is None:
+            return
+
+        opts.removeListener('volume', '{}_{}'.format(self.name, which))
+
+        self.__auximages[which] = None
+        self.__auxopts[  which] = None
+
+        if refreshTexture:
+            self.refreshAuxTexture(which)
+
+
+    def __destroyAuxTexture(self, which):
+        """Destroys the :class:`.ImageTexture` for type ``which``. """
+        tex = self.__auxtextures[which]
+        if tex is not None:
+            glresources.delete(tex.name)
+        self.__auxtextures[which] = None
+
+
+    def refreshAuxTexture(self, which, **kwargs):
+        """Create/re-create an auxillary :class:`.ImageTexture`.
+
+        The previous ``ImageTexture`` (if one exists) is destroyed.  If no
+        :class:`.Image` of type ``which`` is currently registered, a small
+        dummy ``Image`` and ``ImageTexture`` is created.
+
+        :arg which: Name of the auxillary image
+
+        All other arguments are passed through to the
+        :class:`.ImageTexture.__init__` method.
+        """
+
+        self.__destroyAuxTexture(which)
+
+        image = self.__auximages[  which]
+        opts  = self.__auxopts[    which]
+        tex   = self.__auxtextures[which]
+
+        if image is None:
+            textureData    = np.zeros((3, 3, 3), dtype=np.uint8)
+            textureData[:] = 255
+            image          = fslimage.Image(textureData)
+            norm           = None
+        else:
+            norm = image.dataRange
+
+        # by default we use a name which
+        # is not coupled to the aux opts
+        # instance, as the texture may be
+        # sharable.
+        texName = '{}_{}_{}_{}'.format(
+            type(self).__name__, id(self.image), id(image), which)
+
+        # check to see whether the aux
+        # opts object is unsynced from
+        # its parent - if so, we have to
+        # create a dedicated texture
+        if opts is not None:
+            unsynced = (opts.getParent() is None or
+                        not opts.isSyncedToParent('volume'))
+            if unsynced:
+                texName = '{}_unsync_{}'.format(texName, id(opts))
+
+        if opts is not None: volume = opts.index()[3:]
+        else:                volume = 0
+
+        tex = glresources.get(
+            texName,
+            textures.ImageTexture,
+            texName,
+            image,
+            normaliseRange=norm,
+            volume=volume,
+            notify=False,
+            **kwargs)
+
+        self.__auxtextures[which] = tex
