@@ -10,14 +10,13 @@ from a collection of 3D overlays.
 """
 
 import sys
-import copy
 import logging
 
 import numpy     as np
 import OpenGL.GL as gl
 
 import fsl.data.image                    as fslimage
-import fsl.utils.transform               as transform
+import fsl.transform.affine              as affine
 
 import fsleyes.displaycontext.canvasopts as canvasopts
 import fsleyes.gl.slicecanvas            as slicecanvas
@@ -375,9 +374,9 @@ class LightBoxCanvas(slicecanvas.SliceCanvas):
 
             self._offscreenRenderTexture = textures.RenderTexture(
                 '{}_{}'.format(type(self).__name__, id(self)),
-                gl.GL_LINEAR)
+                interp=gl.GL_LINEAR)
 
-            self._offscreenRenderTexture.setSize(768, 768)
+            self._offscreenRenderTexture.shape = 768, 768
 
         # The LightBoxCanvas handles pre-render mode
         # the same way as the SliceCanvas - a separate
@@ -530,7 +529,7 @@ class LightBoxCanvas(slicecanvas.SliceCanvas):
         :attr:`zrange` min/max values.
         """
 
-        slicecanvas.SliceCanvas._overlayBoundsChanged(self)
+        slicecanvas.SliceCanvas._overlayBoundsChanged(self, preserveZoom=False)
 
         self._updateZAxisProperties()
         self._calcNumSlices()
@@ -545,6 +544,9 @@ class LightBoxCanvas(slicecanvas.SliceCanvas):
         be displayed, in display coordinates.
         """
 
+        if self.destroyed:
+            return
+
         opts = self.opts
         xmin = self.displayCtx.bounds.getLo( opts.xax)
         ymin = self.displayCtx.bounds.getLo( opts.yax)
@@ -692,10 +694,7 @@ class LightBoxCanvas(slicecanvas.SliceCanvas):
                 toOrigin[  opts.yax, 3]        = -yoff
                 fromOrigin[opts.yax, 3]        =  yoff
 
-            xform = transform.concat(fromOrigin,
-                                     invert,
-                                     toOrigin,
-                                     xform)
+            xform = affine.concat(fromOrigin, invert, toOrigin, xform)
             invXforms.append(xform)
 
         return invXforms
@@ -740,8 +739,8 @@ class LightBoxCanvas(slicecanvas.SliceCanvas):
 
         for lines in (rowLines, colLines):
             for i in range(0, len(lines), 2):
-                self.getAnnotations().line(lines[i],
-                                           lines[i + 1],
+                self.getAnnotations().line(*lines[i],
+                                           *lines[i + 1],
                                            colour=colour,
                                            width=2)
 
@@ -771,10 +770,11 @@ class LightBoxCanvas(slicecanvas.SliceCanvas):
         # in GL space, the top row is actually the bottom row
         row = self._totalRows - row - 1
 
-        self.getAnnotations().rect((xmin + xlen * col,
-                                    ymin + ylen * row),
+        self.getAnnotations().rect(xmin + xlen * col,
+                                   ymin + ylen * row,
                                    xlen,
                                    ylen,
+                                   filled=False,
                                    colour=(1, 0, 0),
                                    width=2)
 
@@ -823,15 +823,15 @@ class LightBoxCanvas(slicecanvas.SliceCanvas):
             'width'  : 1
         }
 
-        annot.line(xverts[0], xverts[1], **kwargs)
-        annot.line(yverts[0], yverts[1], **kwargs)
+        annot.line(*xverts[0], *xverts[1], **kwargs)
+        annot.line(*yverts[0], *yverts[1], **kwargs)
 
 
     def _draw(self, *a):
         """Draws the current scene to the canvas. """
 
 
-        if self.destroyed():
+        if self.destroyed:
             return
 
         width, height = self.GetSize()
