@@ -25,6 +25,14 @@ class PlotProfile(profiles.Profile):
      - Right click and drag to zoom the plot.
     """
 
+
+    @staticmethod
+    def supportedView():
+        """Returns the :class:`.PlotPanel` class. """
+        from fsleyes.views.plotpanel import PlotPanel
+        return PlotPanel
+
+
     def __init__(self,
                  viewPanel,
                  overlayList,
@@ -53,8 +61,9 @@ class PlotProfile(profiles.Profile):
                                   displayCtx,
                                   modes)
 
-        self.__canvas  = viewPanel.getCanvas()
-        self.__axis    = viewPanel.getAxis()
+        # store our own refs to the matplotlib objects
+        self.__canvas  = viewPanel.canvas.canvas
+        self.__axis    = viewPanel.canvas.axis
 
         # Pan/zoom functionality is actually
         # implemented by the NavigationToolbar2Wx
@@ -75,11 +84,10 @@ class PlotProfile(profiles.Profile):
 
         self.__toolbar.Destroy()
 
-        self.__canvas          = None
-        self.__axis            = None
-        self.__toolbar.canvas  = None
-        self.__toolbar._parent = None
-        self.__toolbar         = None
+        self.__canvas  = None
+        self.__axis    = None
+        self.__toolbar = None
+
         profiles.Profile.destroy(self)
 
 
@@ -99,9 +107,9 @@ class PlotProfile(profiles.Profile):
         xlims = list(self.__axis.get_xlim())
         ylims = list(self.__axis.get_ylim())
 
-        with props.suppress(self._viewPanel, 'limits'):
-            self._viewPanel.limits.x = xlims
-            self._viewPanel.limits.y = ylims
+        with props.suppress(self.viewPanel.canvas, 'limits'):
+            self.viewPanel.canvas.limits.x = xlims
+            self.viewPanel.canvas.limits.y = ylims
 
 
     def _panzoomModeLeftMouseDown(self, ev, canvas, mousePos, canvasPos):
